/* 
 * MQRCP - MQSeries Reason Code Print program
 *
 * "mqrcp.c" Main code module
 *
 * This program accepts a numeric MQI Reason Code value on
 * the command line and displays the corresponding symbolic name.
 * It demonstrates the use of the MQSeries Code to Message conversion
 * functions in the "mqcm.c" module.
 * Its function is almost identical to the new "mqrc" program supplied
 * with MQSeries version 5.2.
 *
 * Copyright 2001 IBM Global Services Australia
 *
 * Source file: L:\MQSeries\UnixUtilities\Source\mqrcp-2.c
 *
 * Revision History
 * ================
 * Version  Date         Author    Changes
 *       1  24 Nov 2000  Glenn     Initial version
 *                       Baddeley <glenbadd@au1.ibm.com>
 *       2  15 Jan 2001  GB        Revision for version 1
 *                                 release 1 of Support Pac MA0K,
 *                                 Display MQCM_RC6MIN and MAX
 */

#include <stdlib.h>
#include <string.h>

/* Define MQLONG */
#include <cmqc.h>

/* MQSeries Code to Message conversion function prototypes */
#include "mqcmh.h"

static char SrcIdent[] = "@(#)MQRCP.C VER=2 DATE=15-Jan-2001";

/****************/
/* Main Program */
/****************/

int main( int ArgC, char *ArgP[] )
{
char StrBuff[64];
MQLONG CodeNum;

  /* Check argument count on command line */
  if( ArgC < 2 )
    {
      printf( "MQSeries Reason Code Print program\n%s\n", SrcIdent );
      printf( "\nCommand line format is:\nmqrcp Code\n\n" );
      printf( "Code may be any decimal number or a hexadecimal number (prefix with X)\n" );
#ifdef MQRC_APPL_FIRST
      printf( "which is a MQI Reason Code (range %d to %d, %d to %d,\n",
              MQRC_APPL_FIRST, MQRC_APPL_LAST, MQCM_RCMIN, MQCM_RCMAX );
      printf( "%d to %d decimal).\n", MQCM_RC6MIN, MQCM_RC6MAX );
#else
      printf( "which is a MQI Reason Code (range %d to %d decimal).\n",
              MQCM_RCMIN, MQCM_RCMAX );
#endif
      exit( 1 );
    };

  /* Convert Reason Code on command line to an integer */
  if( *ArgP[1] == 'X' )
    sscanf( &ArgP[1][1], "%x", &CodeNum );   /* hex */
  else
    sscanf( ArgP[1], "%d", &CodeNum );   /* decimal */

  /* Display the symbolic name for the Reason Code */
  printf( "Reason Code %d X%X\n", CodeNum, CodeNum );
  printf( "%s\n", MQReasonCodeMsg( CodeNum, StrBuff ) );

  exit( 0 );
}

