Attribute VB_Name = "REGC"
'
' REGC - Windows Registry handling routines in Visual Basic 5.0
'
' "vb50lib/regc3.bas"  Global Declarations and Code module
'
' Copyright IBM Global Services Australia 2001
'
' Version   Date         Author     Changes
'       1   10 Oct 1999  Glenn      Initial Version
'                        Baddeley <glenbadd@au1.ibm.com>
'       2   20 Aug 2001  GB         Add description of functions
'       3   24 Aug 2001  GB         Rename functions, Add RegcOpenWriteCreate,
'                                   Add short names for hive keys, Add RegcFmtMsg,
'                                   Add RegcWriteValueLong and RegcWriteValueBin,
'                                   Add RegcPutWindowSize and RegcGetWindowSize
'
Option Explicit

'========== Description of Functions ==========
'
' Sub RegcClose(RChKey As Long)
'   Close a key given the handle to the opened key.
'
' Function RegcOpenRead(ByVal RKHkey As Long, ByVal RKSubKey As String) As Long
'   Open a key for reading and return the handle, zero if error.
'
' Function RegcOpenWrite(ByVal WKHkey As Long, ByVal WKSubKey As String) As Long
'   Open a key for writing and return the handle, zero if error.
'
' Function RegcOpenWriteCreate(ByVal WKHkey As Long, ByVal WKSubKey As String) As Long
'   Open a key for writing, creating it if it doesn't exist, and return the handle, zero if error.
'
' Function RegcQueryValueString(ByVal RQhKey As Long, ByVal RQValue As String, ByVal RQDefaultData As String) As String
'   Returns the string data of the value.
'
' Function RegcQueryValueLong(ByVal RQLhKey As Long, ByVal RQLValue As String, ByVal RQLDefaultData As Long) As Long
'   Returns the long integer data of the value.
'
' Function RegcQueryValueBin(ByVal RQBhKey As Long, ByVal RQBValue As String) As String
'   Returns the binary data of the value.
'
' Function RegcEnumValues(ByVal REhKey As Long, REValue() As String, REData() As String, REType() As Long, REDataLen() As Long) As Integer
'   Enumerate the values of a key, and also the data, data type and data length.
'
' Function RegcWriteValueString(ByVal RWhKey As Long, ByVal RWValue As String, ByVal RWData As String) As Long
'   Write string data for a value.  Returns 0 if success, RegcRetCode if failed.
'
' Function RegcWriteValueLong(ByVal WLhKey As Long, ByVal WLValue As string, ByVal WLData As long) As Long
'   Write long data for a value.  Returns 0 if success, RegcRetCode if failed.
'
' Function RegcWriteValueBin(ByVal WBhKey As Long, ByVal WBValue As String, ByVal WBData As String) As Long
'   Write binary data for a value.  Returns 0 if success, RegcRetCode if failed.
'
' Function RegcDeleteValue(ByVal RDhKey As Long, ByVal RDValue As String) As Long
'   Delete a value.  Returns 0 if success, RegcRetCode if failed.
'
' Function RegcFmtMsg(ByVal RFMErrCode As Long)
'   Return the Windows error message for a Windows error code
'
' Sub RegcGetWindowSize(ByVal GWShKey As Long, ByVal GWSValue As String, GWSArgForm As Form)
'   Set a window size and position from a string in the registry
'
' Sub RegcPutWindowSize(ByVal PWShKey As Long, ByVal PWSValue As String, PWSArgForm As Form)
'   Store a window size and position into a string in the registry
'

'========== Public data ==========

' Data content of a value
Public Const REGCMAXDATASIZE = 512
Public RegcDataBuffer As String * REGCMAXDATASIZE
Public RegcDataLen As Long

' Name of a value
Public Const REGCMAXVALUESIZE = 128
Public RegcValueBuffer As String * REGCMAXVALUESIZE
Public RegcValueLen As Long

' Type of a value. See "REG_xxx" Windows symbolic names.
Public RegcValueType As Long

' Returned code from Windows API registry functions.
' Codes are standard "ERROR_xxx" Windows symbolic names in winerror.h
Public RegcRetCode As Long

'========== Public constants ==========

' Registry hive keys
Public Const HKEY_CLASSES_ROOT = &H80000000
Public Const HKEY_CURRENT_USER = &H80000001
Public Const HKEY_LOCAL_MACHINE = &H80000002
Public Const HKEY_USERS = &H80000003
Public Const HKEY_PERFORMANCE_DATA = &H80000004
Public Const HKEY_CURRENT_CONFIG = &H80000005
Public Const HKEY_DYN_DATA = &H80000006

' Registry hive keys with short names
Public Const HKCR = &H80000000
Public Const HKCU = &H80000001
Public Const HKLM = &H80000002
Public Const HKUS = &H80000003
Public Const HKPDATA = &H80000004
Public Const HKCC = &H80000005
Public Const HKDD = &H80000006

' Registry Data Types (returned in RegcValueType)
Public Const REG_NONE = 0                       ' No value type
Public Const REG_SZ = 1                         ' Unicode nul terminated string
Public Const REG_EXPAND_SZ = 2                  ' Unicode nul terminated string
Public Const REG_BINARY = 3                     ' Free form binary
Public Const REG_DWORD = 4                      ' 32-bit number
Public Const REG_DWORD_LITTLE_ENDIAN = 4        ' 32-bit number (same as REG_DWORD)
Public Const REG_DWORD_BIG_ENDIAN = 5           ' 32-bit number
Public Const REG_LINK = 6                       ' Symbolic Link (unicode)
Public Const REG_MULTI_SZ = 7                   ' Multiple Unicode strings

'========== Local data ==========

Dim hOpenedKey As Long
Dim ErrMsg As String

' General Windows API constants from "winerror.h"

Const ERROR_SUCCESS = 0&
Const ERROR_FILE_NOT_FOUND = 2&
Const ERROR_MORE_DATA = 234&
Const ERROR_NO_MORE_ITEMS = 259&

' Constants for Registry functions

Const SYNCHRONIZE = &H100000
Const READ_CONTROL = &H20000
Const STANDARD_RIGHTS_ALL = &H1F0000
Const STANDARD_RIGHTS_READ = (READ_CONTROL)
Const STANDARD_RIGHTS_WRITE = (READ_CONTROL)

Const KEY_QUERY_VALUE = &H1
Const KEY_SET_VALUE = &H2
Const KEY_CREATE_SUB_KEY = &H4
Const KEY_ENUMERATE_SUB_KEYS = &H8
Const KEY_EVENT = &H1
Const KEY_NOTIFY = &H10
Const KEY_CREATE_LINK = &H20
Const KEY_ALL_ACCESS = ((STANDARD_RIGHTS_ALL Or KEY_QUERY_VALUE Or KEY_SET_VALUE Or KEY_CREATE_SUB_KEY Or KEY_ENUMERATE_SUB_KEYS Or KEY_NOTIFY Or KEY_CREATE_LINK) And (Not SYNCHRONIZE))
Const KEY_READ = ((STANDARD_RIGHTS_READ Or KEY_QUERY_VALUE Or KEY_ENUMERATE_SUB_KEYS Or KEY_NOTIFY) And (Not SYNCHRONIZE))
Const KEY_EXECUTE = ((KEY_READ) And (Not SYNCHRONIZE))
Const KEY_WRITE = ((STANDARD_RIGHTS_WRITE Or KEY_SET_VALUE Or KEY_CREATE_SUB_KEY) And (Not SYNCHRONIZE))

' Key creation options from "winnt.h"
Const REG_OPTION_RESERVED = 0            ' Parameter is reserved
Const REG_OPTION_NON_VOLATILE = 0        ' Key is preserved when system is rebooted
Const REG_OPTION_VOLATILE = 1            ' Key is not preserved when system is rebooted
Const REG_OPTION_CREATE_LINK = 2         ' Created key is a symbolic link
Const REG_OPTION_BACKUP_RESTORE = 4      ' open for backup or restore, special access rules, privilege required
Const REG_OPTION_OPEN_LINK = 8           ' symbolic link

' Constants for FormatMessage
Const FORMAT_MESSAGE_ALLOCATE_BUFFER = &H100
Const FORMAT_MESSAGE_FROM_SYSTEM = &H1000
Const LANG_NEUTRAL = &H0
Const SUBLANG_DEFAULT = &H1

' Windows NT 4.0 Advanced API declarations
' Unused declarations are commented out

' Registry functions
Declare Function RegCloseKey Lib "advapi32.dll" (ByVal hKey As Long) As Long
'Declare Function RegCreateKey Lib "advapi32.dll" Alias "RegCreateKeyA" (ByVal hKey As Long, ByVal lpSubKey As String, phkResult As Long) As Long
Declare Function RegCreateKeyEx Lib "advapi32.dll" Alias "RegCreateKeyExA" (ByVal hKey As Long, ByVal lpSubKey As String, ByVal Reserved As Long, ByVal lpClass As String, ByVal dwOptions As Long, ByVal samDesired As Long, lpSecurityAttributes As Any, phkResult As Long, lpdwDisposition As Long) As Long
'Declare Function RegDeleteKey Lib "advapi32.dll" Alias "RegDeleteKeyA" (ByVal hKey As Long, ByVal lpSubKey As String) As Long
Declare Function RegDeleteValue Lib "advapi32.dll" Alias "RegDeleteValueA" (ByVal hKey As Long, ByVal lpValueName As String) As Long
'Declare Function RegEnumKey Lib "advapi32.dll" Alias "RegEnumKeyA" (ByVal hKey As Long, ByVal dwIndex As Long, ByVal lpName As String, ByVal cbName As Long) As Long
'Declare Function RegEnumKeyEx Lib "advapi32.dll" Alias "RegEnumKeyExA" (ByVal hKey As Long, ByVal dwIndex As Long, ByVal lpName As String, lpcbName As Long, lpReserved As Long, ByVal lpClass As String, lpcbClass As Long, lpftLastWriteTime As FILETIME) As Long
Declare Function RegEnumValue Lib "advapi32.dll" Alias "RegEnumValueA" (ByVal hKey As Long, ByVal dwIndex As Long, ByVal lpValueName As String, lpcbValueName As Long, ByVal lpReserved As Long, lpType As Long, ByVal lpData As String, lpcbData As Long) As Long
Declare Function RegFlushKey Lib "advapi32.dll" (ByVal hKey As Long) As Long
'Declare Function RegLoadKey Lib "advapi32.dll" Alias "RegLoadKeyA" (ByVal hKey As Long, ByVal lpSubKey As String, ByVal lpFile As String) As Long
'Declare Function RegNotifyChangeKeyValue Lib "advapi32.dll" (ByVal hKey As Long, ByVal bWatchSubtree As Long, ByVal dwNotifyFilter As Long, ByVal hEvent As Long, ByVal fAsynchronus As Long) As Long
'Declare Function RegOpenKey Lib "advapi32.dll" Alias "RegOpenKeyA" (ByVal hKey As Long, ByVal lpSubKey As String, phkResult As Long) As Long
Declare Function RegOpenKeyEx Lib "advapi32.dll" Alias "RegOpenKeyExA" (ByVal hKey As Long, ByVal lpSubKey As String, ByVal ulOptions As Long, ByVal samDesired As Long, phkResult As Long) As Long
'Declare Function RegQueryInfoKey Lib "advapi32.dll" Alias "RegQueryInfoKeyA" (ByVal hKey As Long, ByVal lpClass As String, lpcbClass As Long, lpReserved As Long, lpcSubKeys As Long, lpcbMaxSubKeyLen As Long, lpcbMaxClassLen As Long, lpcValues As Long, lpcbMaxValueNameLen As Long, lpcbMaxValueLen As Long, lpcbSecurityDescriptor As Long, lpftLastWriteTime As FILETIME) As Long
'Declare Function RegQueryValue Lib "advapi32.dll" Alias "RegQueryValueA" (ByVal hKey As Long, ByVal lpSubKey As String, ByVal lpValue As String, lpcbValue As Long) As Long
Declare Function RegQueryValueEx Lib "advapi32.dll" Alias "RegQueryValueExA" (ByVal hKey As Long, ByVal lpValueName As String, ByVal lpReserved As Long, lpType As Long, ByVal lpData As String, lpcbData As Long) As Long
Declare Function RegQueryValueExLong Lib "advapi32.dll" Alias "RegQueryValueExA" (ByVal hKey As Long, ByVal lpValueName As String, ByVal lpReserved As Long, lpType As Long, lpData As Long, lpcbData As Long) As Long
'Declare Function RegReplaceKey Lib "advapi32.dll" Alias "RegReplaceKeyA" (ByVal hKey As Long, ByVal lpSubKey As String, ByVal lpNewFile As String, ByVal lpOldFile As String) As Long
'Declare Function RegRestoreKey Lib "advapi32.dll" Alias "RegRestoreKeyA" (ByVal hKey As Long, ByVal lpFile As String, ByVal dwFlags As Long) As Long
'Declare Function RegSaveKey Lib "advapi32.dll" Alias "RegSaveKeyA" (ByVal hKey As Long, ByVal lpFile As String, lpSecurityAttributes As SECURITY_ATTRIBUTES) As Long
'Declare Function RegSetValue Lib "advapi32.dll" Alias "RegSetValueA" (ByVal hKey As Long, ByVal lpSubKey As String, ByVal dwType As Long, ByVal lpData As String, ByVal cbData As Long) As Long
'Declare Function RegSetKeySecurity Lib "advapi32.dll" (ByVal hKey As Long, ByVal SecurityInformation As Long, pSecurityDescriptor As SECURITY_DESCRIPTOR) As Long
Declare Function RegSetValueExS Lib "advapi32.dll" Alias "RegSetValueExA" (ByVal hKey As Long, ByVal lpValueName As String, ByVal Reserved As Long, ByVal dwType As Long, ByVal lpData As String, ByVal cbData As Long) As Long
Declare Function RegSetValueExL Lib "advapi32.dll" Alias "RegSetValueExA" (ByVal hKey As Long, ByVal lpValueName As String, ByVal Reserved As Long, ByVal dwType As Long, lpData As Long, ByVal cbData As Long) As Long
'Declare Function RegUnLoadKey Lib "advapi32.dll" Alias "RegUnLoadKeyA" (ByVal hKey As Long, ByVal lpSubKey As String) As Long

' Windows NT 4.0 Kernel API declarations
Private Declare Function FormatMessage Lib "kernel32" Alias "FormatMessageA" (ByVal dwFlags As Long, lpSource As Any, ByVal dwMessageId As Long, ByVal dwLanguageId As Long, ByVal lpBuffer As String, ByVal nSize As Long, Arguments As Long) As Long

Sub RegcClose(RChKey As Long)

    If RChKey <> 0 Then
        RegcRetCode = RegFlushKey(RChKey)
        If RegcRetCode <> ERROR_SUCCESS Then
            ErrMsg = "Unexpected error from Windows API." & vbCrLf
            ErrMsg = ErrMsg & "RegFlushKey( ) returned " & RegcFmtMsg(RegcRetCode) & vbCrLf
            ErrMsg = ErrMsg & "RChKey=" & RChKey
            MsgBox ErrMsg, vbExclamation, "REGC RegcClose( )"
        End If
        RegcRetCode = RegCloseKey(RChKey)
        If RegcRetCode <> ERROR_SUCCESS Then
            ErrMsg = "Unexpected error from Windows API." & vbCrLf
            ErrMsg = ErrMsg & "RegCloseKey( ) returned " & RegcFmtMsg(RegcRetCode) & vbCrLf
            ErrMsg = ErrMsg & "RChKey=" & RChKey
            MsgBox ErrMsg, vbExclamation, "REGC RegcClose( )"
        Else
            RChKey = 0      ' Success, make the key invalid
        End If
    End If
    
End Sub

Function RegcOpenRead(ByVal RKHkey As Long, ByVal RKSubKey As String) As Long

    RegcRetCode = RegOpenKeyEx(RKHkey, RKSubKey, 0&, KEY_READ, hOpenedKey)
    Select Case RegcRetCode
      Case ERROR_SUCCESS
        RegcOpenRead = hOpenedKey
      Case ERROR_FILE_NOT_FOUND
        RegcOpenRead = 0
      Case Else
        ErrMsg = "Unexpected error from Windows API." & vbCrLf
        ErrMsg = ErrMsg & "RegOpenKeyEx( ) returned " & RegcFmtMsg(RegcRetCode) & vbCrLf
        ErrMsg = ErrMsg & "RKHKey=" & RKHkey & " RKSubKey='" & RKSubKey & "'"
        MsgBox ErrMsg, vbExclamation, "REGC RegcOpenRead( )"
        RegcOpenRead = 0
    End Select
    
End Function

Function RegcOpenWrite(ByVal WKHkey As Long, ByVal WKSubKey As String) As Long

    RegcRetCode = RegOpenKeyEx(WKHkey, WKSubKey, 0&, KEY_ALL_ACCESS, hOpenedKey)
    Select Case RegcRetCode
      Case ERROR_SUCCESS
        RegcOpenWrite = hOpenedKey
      Case ERROR_FILE_NOT_FOUND
        RegcOpenWrite = 0
      Case Else
        ErrMsg = "Unexpected error from Windows API." & vbCrLf
        ErrMsg = ErrMsg & "RegOpenKeyEx( ) returned " & RegcFmtMsg(RegcRetCode) & vbCrLf
        ErrMsg = ErrMsg & "WKHKey=" & WKHkey & " WKSubKey='" & WKSubKey & "'"
        MsgBox ErrMsg, vbExclamation, "REGC RegcOpenWrite( )"
        RegcOpenWrite = 0
    End Select
    
End Function

Function RegcOpenWriteCreate(ByVal WKHkey As Long, ByVal WKSubKey As String) As Long

Dim CrDisposition As Long

    RegcRetCode = RegOpenKeyEx(WKHkey, WKSubKey, 0&, KEY_ALL_ACCESS, hOpenedKey)
    Select Case RegcRetCode
      Case ERROR_SUCCESS
        RegcOpenWriteCreate = hOpenedKey
      Case ERROR_FILE_NOT_FOUND
        GoSub OpenWriteCreate
      Case Else
        ErrMsg = "Unexpected error from Windows API." & vbCrLf
        ErrMsg = ErrMsg & "RegOpenKeyEx( ) returned " & RegcFmtMsg(RegcRetCode) & vbCrLf
        ErrMsg = ErrMsg & "WKHKey=" & WKHkey & " WKSubKey='" & WKSubKey & "'"
        MsgBox ErrMsg, vbExclamation, "REGC RegcOpenWriteCreate( )"
        RegcOpenWriteCreate = 0
    End Select
    
Exit Function

OpenWriteCreate:
        
    RegcRetCode = RegCreateKeyEx(WKHkey, WKSubKey, 0&, "REG_DWORD", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, ByVal 0&, hOpenedKey, CrDisposition)
    Select Case RegcRetCode
      Case ERROR_SUCCESS
        RegcOpenWriteCreate = hOpenedKey
      Case Else
        ErrMsg = "Unexpected error from Windows API." & vbCrLf
        ErrMsg = ErrMsg & "RegCreateKeyEx( ) returned " & RegcFmtMsg(RegcRetCode) & vbCrLf
        ErrMsg = ErrMsg & "WKHKey=" & WKHkey & " WKSubKey='" & WKSubKey & "'"
        MsgBox ErrMsg, vbExclamation, "REGC RegcOpenWriteCreate( )"
        RegcOpenWriteCreate = 0
    End Select

Return
    
End Function

Function RegcQueryValueString(ByVal RQhKey As Long, ByVal RQValue As String, ByVal RQDefaultData As String) As String

    RegcDataLen = REGCMAXDATASIZE
    RegcRetCode = RegQueryValueEx(RQhKey, RQValue, 0&, RegcValueType, RegcDataBuffer, RegcDataLen)
    Select Case RegcRetCode
      Case ERROR_SUCCESS
        Do   ' Dont include trailing nulls
            If RegcDataLen = 0 Then
                RegcQueryValueString = RQDefaultData
                Exit Do
            End If
            If Mid(RegcDataBuffer, RegcDataLen, 1) <> vbNullChar Then
                RegcQueryValueString = Left(RegcDataBuffer, RegcDataLen)
                Exit Do
            End If
            RegcDataLen = RegcDataLen - 1
        Loop
      Case ERROR_FILE_NOT_FOUND
        RegcQueryValueString = RQDefaultData      ' Return the default data
      Case ERROR_MORE_DATA
        RegcDataLen = -RegcDataLen          ' -ve indicates value too long
        RegcQueryValueString = "?DataTooLong"
      Case Else
        ErrMsg = "Unexpected error from Windows API." & vbCrLf
        ErrMsg = ErrMsg & "RegQueryValueEx( ) returned " & RegcFmtMsg(RegcRetCode) & vbCrLf
        ErrMsg = ErrMsg & "RQhKey=" & RQhKey & vbCrLf & "RQValue='" & RQValue & "'" & vbCrLf
        ErrMsg = ErrMsg & "RegcValueType=" & RegcValueType & vbCrLf & "RegcDataLen=" & RegcDataLen
        MsgBox ErrMsg, vbExclamation, "REGC RegcQueryValue( )"
        RegcQueryValueString = "?Error " & RegcRetCode
    End Select

End Function

Function RegcQueryValueLong(ByVal RQLhKey As Long, ByVal RQLValue As String, ByVal RQLDefaultData As Long) As Long

Dim RQLDataLong As Long

    RegcDataLen = 4
    RegcRetCode = RegQueryValueExLong(RQLhKey, RQLValue, 0&, RegcValueType, RQLDataLong, RegcDataLen)
    Select Case RegcRetCode
      Case ERROR_SUCCESS
        RegcQueryValueLong = RQLDataLong
      Case ERROR_FILE_NOT_FOUND
        RegcQueryValueLong = RQLDefaultData      ' Return the default data
      Case Else
        ErrMsg = "Unexpected error from Windows API." & vbCrLf
        ErrMsg = ErrMsg & "RegQueryValueEx( ) returned " & RegcFmtMsg(RegcRetCode) & vbCrLf
        ErrMsg = ErrMsg & "RQLhKey=" & RQLhKey & vbCrLf & "RQLValue='" & RQLValue & "'" & vbCrLf
        ErrMsg = ErrMsg & "RegcValueType=" & RegcValueType & vbCrLf & "RegcDataLen=" & RegcDataLen
        MsgBox ErrMsg, vbExclamation, "REGC RegcQueryValueLong( )"
        RegcQueryValueLong = 0
    End Select

End Function

Function RegcQueryValueBin(ByVal RQBhKey As Long, ByVal RQBValue As String) As String

    RegcDataLen = REGCMAXDATASIZE
    RegcRetCode = RegQueryValueEx(RQBhKey, RQBValue, 0&, RegcValueType, RegcDataBuffer, RegcDataLen)
    Select Case RegcRetCode
      Case ERROR_SUCCESS
        RegcQueryValueBin = Left(RegcDataBuffer, RegcDataLen)
      Case ERROR_FILE_NOT_FOUND
        RegcQueryValueBin = ""    ' Return nothing
        RegcDataLen = -1
      Case Else
        ErrMsg = "Unexpected error from Windows API." & vbCrLf
        ErrMsg = ErrMsg & "RegQueryValueEx( ) returned " & RegcFmtMsg(RegcRetCode) & vbCrLf
        ErrMsg = ErrMsg & "RQBhKey=" & RQBhKey & vbCrLf & "RQBValue='" & RQBValue & "'" & vbCrLf
        ErrMsg = ErrMsg & "RegcValueType=" & RegcValueType & vbCrLf & "RegcDataLen=" & RegcDataLen
        MsgBox ErrMsg, vbExclamation, "REGC RegcQueryValueBin( )"
        RegcQueryValueBin = ""    ' Return nothing
        RegcDataLen = -1
    End Select

End Function

Function RegcEnumValues(ByVal REhKey As Long, REValue() As String, REData() As String, REType() As Long, REDataLen() As Long) As Integer

Dim REIdx As Long

  REIdx = 0
  Do
    RegcValueLen = REGCMAXVALUESIZE
    RegcDataLen = REGCMAXDATASIZE
    RegcRetCode = RegEnumValue(REhKey, REIdx, RegcValueBuffer, RegcValueLen, 0&, RegcValueType, RegcDataBuffer, RegcDataLen)
    Select Case RegcRetCode
      Case ERROR_SUCCESS
        REIdx = REIdx + 1
        REValue(REIdx) = Left(RegcValueBuffer, RegcValueLen)
        Select Case RegcValueType
          Case REG_SZ, REG_EXPAND_SZ
            If RegcValueLen > 0 Then
                Do   ' Dont include trailing nulls on string data
                    If RegcDataLen = 0 Then
                        REData(REIdx) = ""
                        Exit Do
                    End If
                    If Mid(RegcDataBuffer, RegcDataLen, 1) <> vbNullChar Then
                        REData(REIdx) = Left(RegcDataBuffer, RegcDataLen)
                        Exit Do
                    End If
                    RegcDataLen = RegcDataLen - 1
                Loop
            Else
                REData(REIdx) = ""
            End If
          Case Else
            REData(REIdx) = Left(RegcDataBuffer, RegcDataLen)
        End Select
        REType(REIdx) = RegcValueType
        REDataLen(REIdx) = RegcDataLen
      Case ERROR_NO_MORE_ITEMS
        RegcEnumValues = REIdx
        Exit Do
      Case Else
        ErrMsg = "Unexpected error from Windows API." & vbCrLf
        ErrMsg = ErrMsg & "RegEnumValue( ) returned " & RegcFmtMsg(RegcRetCode) & vbCrLf
        ErrMsg = ErrMsg & "REhKey=" & REhKey & vbCrLf & "REIdx=" & REIdx & vbCrLf
        ErrMsg = ErrMsg & "RegcValueType=" & RegcValueType & vbCrLf & "RegcValueLen=" & RegcValueLen & vbCrLf & "RegcDataLen=" & RegcDataLen
        MsgBox ErrMsg, vbExclamation, "REGC RegcEnumValues( )"
        RegcEnumValues = -1
        Exit Do
    End Select
  Loop
    
End Function

Function RegcWriteValueString(ByVal WShKey As Long, ByVal WSValue As String, ByVal WSData As String) As Long

    RegcDataLen = Len(WSData)
    RegcValueType = REG_SZ
    RegcRetCode = RegSetValueExS(WShKey, WSValue, 0&, RegcValueType, WSData, RegcDataLen)
    Select Case RegcRetCode
      Case ERROR_SUCCESS
        RegcWriteValueString = 0
      Case Else
        ErrMsg = "Unexpected error from Windows API." & vbCrLf
        ErrMsg = ErrMsg & "RegSetValueEx( ) returned " & RegcFmtMsg(RegcRetCode) & vbCrLf
        ErrMsg = ErrMsg & "WShKey=" & WShKey & vbCrLf & "WSValue='" & WSValue & "'" & vbCrLf
        ErrMsg = ErrMsg & "WSData='" & WSData & "'" & vbCrLf
        ErrMsg = ErrMsg & "RegcValueType=" & RegcValueType & vbCrLf & "RegcDataLen=" & RegcDataLen
        MsgBox ErrMsg, vbExclamation, "REGC RegcWriteValueString( )"
        RegcWriteValueString = RegcRetCode
    End Select

End Function

Function RegcWriteValueLong(ByVal WLhKey As Long, ByVal WLValue As String, ByVal WLData As Long) As Long

    RegcDataLen = 4
    RegcValueType = REG_DWORD
    RegcRetCode = RegSetValueExL(WLhKey, WLValue, 0&, RegcValueType, WLData, RegcDataLen)
    Select Case RegcRetCode
      Case ERROR_SUCCESS
        RegcWriteValueLong = 0
      Case Else
        ErrMsg = "Unexpected error from Windows API." & vbCrLf
        ErrMsg = ErrMsg & "RegSetValueEx( ) returned " & RegcFmtMsg(RegcRetCode) & vbCrLf
        ErrMsg = ErrMsg & "WLhKey=" & WLhKey & vbCrLf & "WLValue='" & WLValue & "'" & vbCrLf
        ErrMsg = ErrMsg & "RegcValueType=" & RegcValueType & vbCrLf & "RegcDataLen=" & RegcDataLen
        MsgBox ErrMsg, vbExclamation, "REGC RegcWriteValueLong( )"
        RegcWriteValueLong = RegcRetCode
    End Select

End Function

Function RegcWriteValueBin(ByVal WBhKey As Long, ByVal WBValue As String, ByVal WBData As String) As Long

    RegcDataLen = Len(WBData)
    RegcValueType = REG_BINARY
    RegcRetCode = RegSetValueExS(WBhKey, WBValue, 0&, RegcValueType, WBData, RegcDataLen)
    Select Case RegcRetCode
      Case ERROR_SUCCESS
        RegcWriteValueBin = 0
      Case Else
        ErrMsg = "Unexpected error from Windows API." & vbCrLf
        ErrMsg = ErrMsg & "RegSetValueEx( ) returned " & RegcFmtMsg(RegcRetCode) & vbCrLf
        ErrMsg = ErrMsg & "WBhKey=" & WBhKey & vbCrLf & "WBValue='" & WBValue & "'" & vbCrLf
        ErrMsg = ErrMsg & "RegcValueType=" & RegcValueType & vbCrLf & "RegcDataLen=" & RegcDataLen
        MsgBox ErrMsg, vbExclamation, "REGC RegcWriteValueBin( )"
        RegcWriteValueBin = RegcRetCode
    End Select

End Function

Function RegcDeleteValue(ByVal RDhKey As Long, ByVal RDValue As String) As Long

    RegcRetCode = RegDeleteValue(RDhKey, RDValue)
    Select Case RegcRetCode
      Case ERROR_SUCCESS
        RegcDeleteValue = 0
      Case Else
        ErrMsg = "Unexpected error from Windows API." & vbCrLf
        ErrMsg = ErrMsg & "RegDeleteValue( ) returned " & RegcFmtMsg(RegcRetCode) & vbCrLf
        ErrMsg = ErrMsg & "RDhKey=" & RDhKey & vbCrLf & "RDValue='" & RDValue & "'" & vbCrLf
        MsgBox ErrMsg, vbExclamation, "REGC RegcDeleteValue( )"
        RegcDeleteValue = RegcRetCode
    End Select

End Function

Function RegcFmtMsg(ByVal RFMErrCode As Long)

Const MSGBUFFERSIZE = 256
Dim MsgBuffer As String * MSGBUFFERSIZE
Dim ChPos As Integer, RetLen As Long
    
    RetLen = FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, ByVal 0&, RFMErrCode, LANG_NEUTRAL, MsgBuffer, MSGBUFFERSIZE, ByVal 0&)
    If RetLen < 1 Then
        ErrMsg = "Unexpected error from Windows API." & vbCrLf
        ErrMsg = ErrMsg & "FormatMessage( ) returned " & RetLen & vbCrLf
        ErrMsg = ErrMsg & "RFMErrCode=" & RFMErrCode & vbCrLf
        MsgBox ErrMsg, vbExclamation, "REGC RegcFmtMsg( )"
        RegcFmtMsg = RFMErrCode & ": FormatMessage() failed!"
    ElseIf Mid(MsgBuffer, RetLen + 1, 1) = vbNullChar Then
        If Mid(MsgBuffer, RetLen - 1, 2) = vbCrLf Then
            RegcFmtMsg = RFMErrCode & ": " & Left(MsgBuffer, RetLen - 2)
        Else
            RegcFmtMsg = RFMErrCode & ": " & Left(MsgBuffer, RetLen)
        End If
    Else
        RegcFmtMsg = RFMErrCode & ": " & MsgBuffer
    End If
    
End Function

Sub RegcGetWindowSize(ByVal GWShKey As Long, ByVal GWSValue As String, GWSArgForm As Form)

Const VALSEP = ","
Const DEFVAL = "0,0,0,0"
Dim KeyValues As String, PrevPos As Integer, ChPos As Integer, Idx As Integer

    KeyValues = RegcQueryValueString(GWShKey, GWSValue, DEFVAL) & VALSEP
    If RegcRetCode Then Exit Sub
    ReDim WinSize(1 To 4) As Single
    PrevPos = 1
    For Idx = 1 To 4
        ChPos = InStr(PrevPos, KeyValues, VALSEP)
        WinSize(Idx) = CSng(Mid(KeyValues, PrevPos, ChPos - PrevPos))
        PrevPos = ChPos + 1
    Next Idx
    If WinSize(4) <> 0 Then
        GWSArgForm.Move WinSize(1), WinSize(2)
        GWSArgForm.Width = WinSize(3)
        GWSArgForm.Height = WinSize(4)
    End If

End Sub

Sub RegcPutWindowSize(ByVal PWShKey As Long, ByVal PWSValue As String, PWSArgForm As Form)

Const VALSEP = ","
Dim PWSData As String

    If PWSArgForm.WindowState = vbNormal Then
        PWSData = PWSArgForm.Left & VALSEP & PWSArgForm.Top & VALSEP & PWSArgForm.Width & VALSEP & PWSArgForm.Height
        RegcWriteValueString PWShKey, PWSValue, PWSData
    End If
    
End Sub

